@@ -1,12 +1,8 @@
-use std::error::Error;
-use std::fmt;
 use std::mem;
 
 use crate::links::Links;
 use crate::non_max::NonMaxUsize;
-use crate::Kind;
-use crate::Span;
-use crate::Tree;
+use crate::{Kind, Span, Tree, TreeError};
 
 /// The identifier of a node as returned by functions such as
 /// [TreeBuilder::open] or [TreeBuilder::token].
@@ -17,95 +13,6 @@ use crate::Tree;
 #[repr(transparent)]
 pub struct Id(NonMaxUsize);
 
-/// Errors raised while building a tree.
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
-#[non_exhaustive]
-pub enum TreeBuilderError {
-    /// Error raised by [TreeBuilder::close] if there currently is no node being
-    /// built.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use syntree::{TreeBuilder, TreeBuilderError};
-    ///
-    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
-    /// let mut tree = TreeBuilder::new();
-    ///
-    /// tree.open("root");
-    /// tree.close()?;
-    ///
-    /// // Syntax::Root and Syntax::Child is left open.
-    /// assert_eq!(tree.close(), Err(TreeBuilderError::CloseError));
-    /// # Ok(()) }
-    /// ```
-    CloseError,
-    /// Error raised by [TreeBuilder::build] if the tree isn't correctly
-    /// balanced.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use syntree::{TreeBuilder, TreeBuilderError};
-    ///
-    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
-    /// let mut tree = TreeBuilder::new();
-    ///
-    /// tree.open("number");
-    /// tree.token("lit", 3);
-    /// tree.close()?;
-    ///
-    /// tree.open("number");
-    ///
-    /// // Syntax::Number is left open.
-    /// assert_eq!(tree.build(), Err(TreeBuilderError::BuildError));
-    /// # Ok(()) }
-    /// ```
-    BuildError,
-    /// Error raised by [TreeBuilder::close_at] if we're not trying to close at
-    /// a sibling node.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use syntree::{TreeBuilder, TreeBuilderError};
-    ///
-    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
-    /// let mut tree = TreeBuilder::new();
-    ///
-    /// let c = tree.checkpoint();
-    ///
-    /// tree.open("child");
-    /// tree.token("token", 3);
-    ///
-    /// let result = tree.close_at(c, "operation");
-    /// assert_eq!(result, Err(TreeBuilderError::CloseAtError));
-    /// # Ok(()) }
-    /// ```
-    CloseAtError,
-}
-
-impl Error for TreeBuilderError {}
-
-impl fmt::Display for TreeBuilderError {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        match self {
-            TreeBuilderError::CloseError => {
-                write!(f, "no node being built")
-            }
-            TreeBuilderError::BuildError => {
-                write!(f, "tree is currently being built")
-            }
-            TreeBuilderError::CloseAtError => {
-                write!(
-                    f,
-                    "trying to close a node which is not a sibling of the checkpoint being closed"
-                )
-            }
-        }
-    }
-}
-
 /// A builder for a [Tree].
 ///
 /// This maintains a stack of nodes being built which has to be balanced with
@@ -150,7 +57,6 @@ pub struct TreeBuilder<T> {
     cursor: usize,
 }
 
-/// Build a new syntax tree.
 impl<T> TreeBuilder<T> {
     /// Construct a new tree.
     ///
@@ -195,7 +101,6 @@ impl<T> TreeBuilder<T> {
             cursor: 0,
         }
     }
-
     /// Start a node with the given `data`.
     ///
     /// This pushes a new link with the given type onto the stack which links
@@ -230,7 +135,7 @@ impl<T> TreeBuilder<T> {
     /// End a node being built.
     ///
     /// This call must be balanced with a prior call to [TreeBuilder::open]. If
-    /// not this will result in an [TreeBuilderError::CloseError] being raised.
+    /// not this will result in an [TreeError::CloseError] being raised.
     ///
     /// This will pop a value of the stack, and set that value as the next
     /// sibling which will be used with [TreeBuilder::open].
@@ -254,10 +159,10 @@ impl<T> TreeBuilder<T> {
     /// tree.close()?;
     /// # Ok(()) }
     /// ```
-    pub fn close(&mut self) -> Result<(), TreeBuilderError> {
+    pub fn close(&mut self) -> Result<(), TreeError> {
         let head = match self.parents.pop() {
             Some(head) => head,
-            None => return Err(TreeBuilderError::CloseError),
+            None => return Err(TreeError::CloseError),
         };
 
         self.sibling = Some(head);
@@ -275,9 +180,9 @@ impl<T> TreeBuilder<T> {
         Ok(())
     }
 
-    /// Declare a token with the specified `data` and a corresponding `span`.
+    /// Declare a token with the specified `value` and a corresponding `len`.
     ///
-    /// A token is always a terminating node that has no children.
+    /// A token is always a terminating element without children.
     ///
     /// # Examples
     ///
@@ -293,10 +198,10 @@ impl<T> TreeBuilder<T> {
     ///
     /// # Ok(()) }
     /// ```
-    pub fn token(&mut self, data: T, len: usize) -> Id {
+    pub fn token(&mut self, value: T, len: usize) -> Id {
         let start = self.cursor;
         self.cursor = self.cursor.checked_add(len).expect("cursor out of bounds");
-        let id = self.insert(data, Kind::Token, Span::new(start, self.cursor));
+        let id = self.insert(value, Kind::Token, Span::new(start, self.cursor));
         self.sibling = Some(id);
         Id(id)
     }
@@ -306,7 +211,7 @@ impl<T> TreeBuilder<T> {
     /// # Panics
     ///
     /// This panics if the number of nodes are too many to fit in a vector on
-    /// your architecture. This corresponds to [usize::max_value()].
+    /// your architecture. This corresponds to [usize::MAX].
     ///
     /// # Examples
     ///
@@ -314,30 +219,25 @@ impl<T> TreeBuilder<T> {
     /// use syntree::TreeBuilder;
     ///
     /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
-    /// let mut b = TreeBuilder::new();
+    /// let mut tree = TreeBuilder::new();
     ///
-    /// let c = b.checkpoint();
+    /// let c = tree.checkpoint();
+    /// tree.open("child");
+    /// tree.token("lit", 3);
+    /// tree.close()?;
+    /// tree.close_at(c, "root")?;
     ///
-    /// b.open("number");
-    /// b.token("lit", 1);
-    /// b.close()?;
+    /// let tree = tree.build()?;
     ///
-    /// b.token("whitespace", 3);
+    /// let expected = syntree::tree! {
+    ///     "root" => {
+    ///         "child" => {
+    ///             ("lit", 3)
+    ///         }
+    ///     }
+    /// };
     ///
-    /// b.open("number");
-    /// b.token("lit", 2);
-    /// b.token("lit", 2);
-    /// b.close()?;
-    ///
-    /// b.close_at(c, "root")?;
-    ///
-    /// let tree = b.build()?;
-    ///
-    /// let root = tree.first().ok_or("missing root")?;
-    ///
-    /// assert_eq!(*root.value(), "root");
-    /// assert_eq!(root.children().count(), 3);
-    /// assert_eq!(root.children().without_tokens().count(), 2);
+    /// assert_eq!(tree, expected);
     /// # Ok(()) }
     /// ```
     pub fn checkpoint(&self) -> Id {
@@ -347,7 +247,7 @@ impl<T> TreeBuilder<T> {
     /// Insert a node that wraps from the given checkpointed location.
     ///
     /// The checkpoint being closed *must* be a sibling. Otherwise a
-    /// [TreeBuilderError::CloseAtError] will be raised.
+    /// [TreeError::CloseAtError] will be raised.
     ///
     /// # Examples
     ///
@@ -355,13 +255,13 @@ impl<T> TreeBuilder<T> {
     /// use syntree::TreeBuilder;
     ///
     /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
-    /// let mut b = TreeBuilder::new();
+    /// let mut tree = TreeBuilder::new();
     ///
-    /// let c = b.checkpoint();
-    /// b.token("lit", 3);
-    /// b.close_at(c, "root")?;
+    /// let c = tree.checkpoint();
+    /// tree.token("lit", 3);
+    /// tree.close_at(c, "root")?;
     ///
-    /// let tree = b.build()?;
+    /// let tree = tree.build()?;
     ///
     /// let expected = syntree::tree! {
     ///     "root" => {
@@ -379,23 +279,23 @@ impl<T> TreeBuilder<T> {
     /// use syntree::TreeBuilder;
     ///
     /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
-    /// let mut b = TreeBuilder::new();
+    /// let mut tree = TreeBuilder::new();
     ///
-    /// let c = b.checkpoint();
+    /// let c = tree.checkpoint();
     ///
-    /// b.open("number");
-    /// b.token("lit", 3);
-    /// b.close()?;
+    /// tree.open("number");
+    /// tree.token("lit", 3);
+    /// tree.close()?;
     ///
-    /// b.token("whitespace", 1);
+    /// tree.token("whitespace", 1);
     ///
-    /// b.open("number");
-    /// b.token("lit", 2);
-    /// b.close()?;
+    /// tree.open("number");
+    /// tree.token("lit", 2);
+    /// tree.close()?;
     ///
-    /// b.close_at(c, "root")?;
+    /// tree.close_at(c, "root")?;
     ///
-    /// let tree = b.build()?;
+    /// let tree = tree.build()?;
     ///
     /// let expected = syntree::tree! {
     ///     "root" => {
@@ -412,7 +312,37 @@ impl<T> TreeBuilder<T> {
     /// assert_eq!(tree, expected);
     /// # Ok(()) }
     /// ```
-    pub fn close_at(&mut self, id: Id, data: T) -> Result<Id, TreeBuilderError> {
+    ///
+    /// Adding a token after a checkpoint:
+    ///
+    /// ```
+    /// use syntree::TreeBuilder;
+    ///
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
+    /// let mut tree = TreeBuilder::new();
+    ///
+    /// let c = tree.checkpoint();
+    /// tree.open("child");
+    /// tree.token("lit", 3);
+    /// tree.close()?;
+    /// tree.close_at(c, "root")?;
+    /// tree.token("sibling", 3);
+    ///
+    /// let tree = tree.build()?;
+    ///
+    /// let expected = syntree::tree! {
+    ///     "root" => {
+    ///         "child" => {
+    ///             ("lit", 3)
+    ///         }
+    ///     },
+    ///     ("sibling", 3)
+    /// };
+    ///
+    /// assert_eq!(tree, expected);
+    /// # Ok(()) }
+    /// ```
+    pub fn close_at(&mut self, id: Id, data: T) -> Result<Id, TreeError> {
         let child = NonMaxUsize::new(self.tree.len()).expect("ran out of ids");
 
         let links = match self.tree.get_mut(id.0) {
@@ -424,6 +354,12 @@ impl<T> TreeBuilder<T> {
             }
         };
 
+        // We attempt to restructure the current node, unless it is an immediate
+        // and sole sibling - which it is if it was just inserted into
+        // next.sibling.
+        //
+        // The primary purpose of restructuring is to calculate the span of the
+        // range of nodes we are wrapping.
         let needs_restructuring = self.sibling != Some(id.0);
 
         let removed = mem::replace(
@@ -438,32 +374,8 @@ impl<T> TreeBuilder<T> {
         );
 
         if needs_restructuring {
-            // Adjust span to encapsulate all children and check that we just
-            // inserted ourselves in the right location.
             let start = links.span.start;
-
-            let sibling = self
-                .tree
-                .node_at(self.sibling)
-                .ok_or(TreeBuilderError::CloseAtError)?;
-
-            if let Some(mut node) = self.tree.node_at(removed.next) {
-                while let Some(next) = node.next() {
-                    node = next;
-                }
-
-                let span = Span::new(start, node.span().end);
-
-                if !sibling.is_same(&node) {
-                    return Err(TreeBuilderError::CloseAtError);
-                }
-
-                if let Some(node) = self.tree.get_mut(id.0) {
-                    node.span = span;
-                }
-            } else if !sibling.is_same_as_links(&removed) {
-                return Err(TreeBuilderError::CloseAtError);
-            }
+            self.restructure_close_at(id.0, start, &removed)?;
         }
 
         self.tree.push(removed);
@@ -473,10 +385,46 @@ impl<T> TreeBuilder<T> {
         Ok(Id(id.0))
     }
 
+    // Adjust span to encapsulate all children and check that we just inserted
+    // the checkpointed node in the right location which should be the tail
+    // sibling of the replaced node.
+    fn restructure_close_at(
+        &mut self,
+        id: NonMaxUsize,
+        start: usize,
+        removed: &Links<T>,
+    ) -> Result<(), TreeError> {
+        let sibling = self
+            .tree
+            .node_at(self.sibling)
+            .ok_or(TreeError::CloseAtError)?;
+
+        let node = self
+            .tree
+            .node_at(removed.next)
+            .and_then(|n| n.siblings().last());
+
+        if let Some(node) = node {
+            let span = Span::new(start, node.span().end);
+
+            if !sibling.is_same(&node) {
+                return Err(TreeError::CloseAtError);
+            }
+
+            if let Some(node) = self.tree.get_mut(id) {
+                node.span = span;
+            }
+        } else if !sibling.is_same_as_links(removed) {
+            return Err(TreeError::CloseAtError);
+        }
+
+        Ok(())
+    }
+
     /// Build a [Tree] from the current state of the builder.
     ///
     /// This requires the stack in the builder to be empty. Otherwise a
-    /// [TreeBuilderError::BuildError] will be raised.
+    /// [TreeError::BuildError] will be raised.
     ///
     /// # Examples
     ///
@@ -508,7 +456,7 @@ impl<T> TreeBuilder<T> {
     /// If a tree is unbalanced during construction, building will fail with an error:
     ///
     /// ```
-    /// use syntree::{TreeBuilderError, Span, TreeBuilder};
+    /// use syntree::{TreeError, Span, TreeBuilder};
     ///
     /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
     /// let mut tree = TreeBuilder::new();
@@ -520,12 +468,12 @@ impl<T> TreeBuilder<T> {
     /// tree.open("number");
     ///
     /// // "number" is left open.
-    /// assert!(matches!(tree.build(), Err(TreeBuilderError::BuildError)));
+    /// assert!(matches!(tree.build(), Err(TreeError::BuildError)));
     /// # Ok(()) }
     /// ```
-    pub fn build(self) -> Result<Tree<T>, TreeBuilderError> {
+    pub fn build(self) -> Result<Tree<T>, TreeError> {
         if !self.parents.is_empty() {
-            return Err(TreeBuilderError::BuildError);
+            return Err(TreeError::BuildError);
         }
 
         Ok(self.tree)
