@@ -1,9 +1,10 @@
 use std::fmt;
+use std::ptr;
 
 use crate::links::Links;
 use crate::non_max::NonMaxUsize;
 use crate::tree::Kind;
-use crate::{Children, Span, Walk, WalkEvents};
+use crate::{Nodes, Span, Walk, WalkEvents};
 
 /// A node in the tree.
 pub struct Node<'a, T> {
@@ -12,7 +13,7 @@ pub struct Node<'a, T> {
 }
 
 impl<'a, T> Node<'a, T> {
-    pub(crate) fn new(node: &'a Links<T>, tree: &'a [Links<T>]) -> Self {
+    pub(crate) const fn new(node: &'a Links<T>, tree: &'a [Links<T>]) -> Self {
         Self { links: node, tree }
     }
 
@@ -20,23 +21,68 @@ impl<'a, T> Node<'a, T> {
     ///
     /// This is a cheap pointer comparison.
     pub(crate) fn is_same(&self, other: &Node<'a, T>) -> bool {
-        std::ptr::eq(self.links, other.links)
+        ptr::eq(self.links, other.links)
     }
 
     /// Test if this node is the same as another set of links.
     ///
     /// This is a cheap pointer comparison.
     pub(crate) fn is_same_as_links(&self, links: &Links<T>) -> bool {
-        std::ptr::eq(self.links, links)
+        ptr::eq(self.links, links)
     }
 
     /// Access the data associated with the node.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
+    /// let tree = syntree::tree! {
+    ///     "root" => {
+    ///         ("number", 5),
+    ///         ("ident", 3),
+    ///     }
+    /// };
+    ///
+    /// let root = tree.first().ok_or("missing root")?;
+    /// assert_eq!(*root.value(), "root");
+    ///
+    /// let number = root.first().ok_or("missing number")?;
+    /// assert_eq!(*number.value(), "number");
+    ///
+    /// let ident = number.next().ok_or("missing ident")?;
+    /// assert_eq!(*ident.value(), "ident");
+    /// # Ok(()) }
+    /// ```
     pub fn value(&self) -> &'a T {
         &self.links.data
     }
 
     /// Access the kind of the node.
-    pub fn kind(&self) -> Kind {
+    ///
+    /// Terminating nodes are [Kind::Token] and intermediary nodes are
+    /// [Kind::Node].
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use syntree::Kind;
+    ///
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
+    /// let tree = syntree::tree! {
+    ///     "root" => {
+    ///         ("number", 5),
+    ///         ("ident", 3),
+    ///     }
+    /// };
+    ///
+    /// let root = tree.first().ok_or("missing root")?;
+    /// assert_eq!(root.kind(), Kind::Node);
+    ///
+    /// assert!(root.children().all(|n| matches!(n.kind(), Kind::Token)));
+    /// # Ok(()) }
+    /// ```
+    pub const fn kind(&self) -> Kind {
         self.links.kind
     }
 
@@ -70,7 +116,7 @@ impl<'a, T> Node<'a, T> {
     /// assert_eq!(root2.span(), Span::new(8, 13));
     /// # Ok(()) }
     /// ```
-    pub fn span(&self) -> Span {
+    pub const fn span(&self) -> Span {
         self.links.span
     }
 
@@ -95,15 +141,22 @@ impl<'a, T> Node<'a, T> {
     /// assert!(!last.is_empty());
     /// # Ok(()) }
     /// ```
-    pub fn is_empty(&self) -> bool {
+    pub const fn is_empty(&self) -> bool {
         self.links.first.is_none()
     }
 
+    /// Get an iterator over the siblings of this node, including itself.
+    ///
+    /// See [Nodes] for documentation.
+    pub fn siblings(&self) -> Nodes<'a, T> {
+        Nodes::new(Some(*self))
+    }
+
     /// Get an iterator over the children of this node.
     ///
-    /// See [Children] for documentation.
-    pub fn children(&self) -> Children<'a, T> {
-        Children::new(self.tree, self.links.first)
+    /// See [Nodes] for documentation.
+    pub fn children(&self) -> Nodes<'a, T> {
+        Nodes::new(self.first())
     }
 
     /// Walk the subtree forward starting with the first child of the current
@@ -111,15 +164,15 @@ impl<'a, T> Node<'a, T> {
     ///
     /// See [Walk] for documentation.
     pub fn walk(&self) -> Walk<'a, T> {
-        Walk::new(self.tree, self.links.first)
+        Walk::new(self.first())
     }
 
     /// Walk the node forwards in a depth-first fashion emitting events
     /// indicating how the rest of the tree is being traversed.
     ///
     /// See [WalkEvents] for documentation.
-    pub fn walk_events(&self) -> WalkEvents<'_, T> {
-        WalkEvents::new(self.tree, self.links.first)
+    pub fn walk_events(&self) -> WalkEvents<'a, T> {
+        WalkEvents::new(self.first())
     }
 
     /// Get the first child node.
@@ -184,8 +237,8 @@ impl<'a, T> Node<'a, T> {
         self.node_at(self.links.next)
     }
 
-    fn node_at(&self, index: Option<NonMaxUsize>) -> Option<Node<'a, T>> {
-        let cur = self.tree.get(index?.get())?;
+    fn node_at(&self, id: Option<NonMaxUsize>) -> Option<Node<'a, T>> {
+        let cur = self.tree.get(id?.get())?;
 
         Some(Self {
             links: cur,
