@@ -2,7 +2,7 @@ use std::fmt;
 
 use crate::links::Links;
 use crate::non_max::NonMaxUsize;
-use crate::{Children, Node, Walk, WalkEvents};
+use crate::{Node, Nodes, Walk, WalkEvents};
 
 /// The kind of a node in the [Tree].
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
@@ -46,16 +46,16 @@ impl<T> Tree<T> {
 
     /// Get all root nodes in the tree.
     ///
-    /// See [Children] for documentation.
-    pub fn children(&self) -> Children<'_, T> {
-        Children::new(self.tree.as_ref(), NonMaxUsize::new(0))
+    /// See [Nodes] for documentation.
+    pub fn children(&self) -> Nodes<'_, T> {
+        Nodes::new(self.first())
     }
 
     /// Walk the tree forwards in a depth-first fashion visiting every node once.
     ///
     /// See [Walk] for documentation.
     pub fn walk(&self) -> Walk<'_, T> {
-        Walk::new(self.tree.as_ref(), NonMaxUsize::new(0))
+        Walk::new(self.first())
     }
 
     /// Walk the tree forwards in a depth-first fashion emitting events
@@ -63,7 +63,7 @@ impl<T> Tree<T> {
     ///
     /// See [WalkEvents] for documentation.
     pub fn walk_events(&self) -> WalkEvents<'_, T> {
-        WalkEvents::new(self.tree.as_ref(), NonMaxUsize::new(0))
+        WalkEvents::new(self.first())
     }
 
     /// Get the first child node in the tree.
